package sun.awt.image;

import java.awt.image.WritableRaster;
import java.awt.image.Raster;
import java.awt.image.RasterFormatException;
import java.awt.image.MultiPixelPackedSampleModel;
import java.awt.image.DataBufferByte;
import java.awt.image.DataBuffer;
import java.awt.Rectangle;
import java.awt.Point;
import java.awt.image.SampleModel;

public class BytePackedRaster extends SunWritableRaster
{
    int dataBitOffset;
    int scanlineStride;
    int pixelBitStride;
    int bitMask;
    byte[] data;
    int shiftOffset;
    int type;
    private int maxX;
    private int maxY;
    
    private static native void initIDs();
    
    public BytePackedRaster(SampleModel sampleModel, Point point) {
        this(sampleModel, sampleModel.createDataBuffer(), new Rectangle(point.x, point.y, sampleModel.getWidth(), sampleModel.getHeight()), point, null);
    }
    
    public BytePackedRaster(SampleModel sampleModel, DataBuffer dataBuffer, Point point) {
        this(sampleModel, dataBuffer, new Rectangle(point.x, point.y, sampleModel.getWidth(), sampleModel.getHeight()), point, null);
    }
    
    public BytePackedRaster(SampleModel sampleModel, DataBuffer dataBuffer, Rectangle rectangle, Point point, BytePackedRaster bytePackedRaster) {
        super(sampleModel, dataBuffer, rectangle, point, bytePackedRaster);
        this.maxX = this.minX + this.width;
        this.maxY = this.minY + this.height;
        if (!(dataBuffer instanceof DataBufferByte)) {
            throw new RasterFormatException("BytePackedRasters must havebyte DataBuffers");
        }
        DataBufferByte dataBufferByte = (DataBufferByte)dataBuffer;
        this.data = SunWritableRaster.stealData(dataBufferByte, 0);
        if (dataBufferByte.getNumBanks() != 1) {
            throw new RasterFormatException("DataBuffer for BytePackedRasters must only have 1 bank.");
        }
        int offset = dataBufferByte.getOffset();
        if ((sampleModel instanceof MultiPixelPackedSampleModel)) {
            MultiPixelPackedSampleModel multiPixelPackedSampleModel = (MultiPixelPackedSampleModel)sampleModel;
            this.type = 11;
            this.pixelBitStride = multiPixelPackedSampleModel.getPixelBitStride();
            if (this.pixelBitStride != 1 && this.pixelBitStride != 2 && this.pixelBitStride != 4) {
                throw new RasterFormatException("BytePackedRasters must have a bit depth of 1, 2, or 4");
            }
            this.scanlineStride = multiPixelPackedSampleModel.getScanlineStride();
            this.dataBitOffset = multiPixelPackedSampleModel.getDataBitOffset() + offset * 8;
            this.dataBitOffset += (rectangle.x - point.x) * this.pixelBitStride + (rectangle.y - point.y) * this.scanlineStride * 8;
            this.bitMask = (1 << this.pixelBitStride) - 1;
            this.shiftOffset = 8 - this.pixelBitStride;
            this.verify(false);
            return;
        }
        else {
            throw new RasterFormatException("BytePackedRasters must haveMultiPixelPackedSampleModel");
        }
    }
    
    public int getDataBitOffset() {
        return this.dataBitOffset;
    }
    
    public int getScanlineStride() {
        return this.scanlineStride;
    }
    
    public int getPixelBitStride() {
        return this.pixelBitStride;
    }
    
    public byte[] getDataStorage() {
        return this.data;
    }
    
    @Override
    public Object getDataElements(int n, int n2, Object o) {
        if (n < this.minX || n2 < this.minY || n >= this.maxX || n2 >= this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        byte[] array;
        if (o == null) {
            array = new byte[this.numDataElements];
        }
        else {
            array = (byte[])(byte[])o;
        }
        int n3 = this.dataBitOffset + (n - this.minX) * this.pixelBitStride;
        array[0] = (byte)((this.data[(n2 - this.minY) * this.scanlineStride + (n3 >> 3)] & 0xFF) >> this.shiftOffset - (n3 & 0x7) & this.bitMask);
        return array;
    }
    
    @Override
    public Object getDataElements(int n, int n2, int n3, int n4, Object o) {
        return this.getByteData(n, n2, n3, n4, (byte[])(byte[])o);
    }
    
    public Object getPixelData(int n, int n2, int n3, int n4, Object o) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        byte[] array;
        if (o == null) {
            array = new byte[this.numDataElements * n3 * n4];
        }
        else {
            array = (byte[])(byte[])o;
        }
        int pixelBitStride = this.pixelBitStride;
        int n5 = this.dataBitOffset + (n - this.minX) * pixelBitStride;
        int n6 = (n2 - this.minY) * this.scanlineStride;
        int n7 = 0;
        byte[] data = this.data;
        for (int i = 0; i < n4; n6 += this.scanlineStride, ++i) {
            for (int j = 0; j < n3; ++j) {
                array[n7++] = (byte)(this.bitMask & data[n6 + (n5 >> 3)] >> this.shiftOffset - (n5 & 0x7));
                n5 += pixelBitStride;
            }
        }
        return array;
    }
    
    public byte[] getByteData(int n, int n2, int n3, int n4, int n5, byte[] array) {
        return this.getByteData(n, n2, n3, n4, array);
    }
    
    public byte[] getByteData(int n, int n2, int n3, int n4, byte[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        if (array == null) {
            array = new byte[n3 * n4];
        }
        int pixelBitStride = this.pixelBitStride;
        int n5 = this.dataBitOffset + (n - this.minX) * pixelBitStride;
        int n6 = (n2 - this.minY) * this.scanlineStride;
        int n7 = 0;
        byte[] data = this.data;
        for (int i = 0; i < n4; n6 += this.scanlineStride, ++i) {
            int n8;
            int j;
            for (n8 = n5, j = 0; j < n3 && (n8 & 0x7) != 0x0; n8 += pixelBitStride, ++j) {
                array[n7++] = (byte)(this.bitMask & data[n6 + (n8 >> 3)] >> this.shiftOffset - (n8 & 0x7));
            }
            int n9 = n6 + (n8 >> 3);
            switch (pixelBitStride) {
                case 1: {
                    while (j < n3 - 7) {
                        int n10 = data[n9++];
                        array[n7++] = (byte)(n10 >> 7 & 0x1);
                        array[n7++] = (byte)(n10 >> 6 & 0x1);
                        array[n7++] = (byte)(n10 >> 5 & 0x1);
                        array[n7++] = (byte)(n10 >> 4 & 0x1);
                        array[n7++] = (byte)(n10 >> 3 & 0x1);
                        array[n7++] = (byte)(n10 >> 2 & 0x1);
                        array[n7++] = (byte)(n10 >> 1 & 0x1);
                        array[n7++] = (byte)(n10 & 0x1);
                        n8 += 8;
                        j += 8;
                    }
                    break;
                }
                case 2: {
                    while (j < n3 - 7) {
                        int n11 = data[n9++];
                        array[n7++] = (byte)(n11 >> 6 & 0x3);
                        array[n7++] = (byte)(n11 >> 4 & 0x3);
                        array[n7++] = (byte)(n11 >> 2 & 0x3);
                        array[n7++] = (byte)(n11 & 0x3);
                        int n12 = data[n9++];
                        array[n7++] = (byte)(n12 >> 6 & 0x3);
                        array[n7++] = (byte)(n12 >> 4 & 0x3);
                        array[n7++] = (byte)(n12 >> 2 & 0x3);
                        array[n7++] = (byte)(n12 & 0x3);
                        n8 += 16;
                        j += 8;
                    }
                    break;
                }
                case 4: {
                    while (j < n3 - 7) {
                        int n13 = data[n9++];
                        array[n7++] = (byte)(n13 >> 4 & 0xF);
                        array[n7++] = (byte)(n13 & 0xF);
                        int n14 = data[n9++];
                        array[n7++] = (byte)(n14 >> 4 & 0xF);
                        array[n7++] = (byte)(n14 & 0xF);
                        int n15 = data[n9++];
                        array[n7++] = (byte)(n15 >> 4 & 0xF);
                        array[n7++] = (byte)(n15 & 0xF);
                        int n16 = data[n9++];
                        array[n7++] = (byte)(n16 >> 4 & 0xF);
                        array[n7++] = (byte)(n16 & 0xF);
                        n8 += 32;
                        j += 8;
                    }
                    break;
                }
            }
            while (j < n3) {
                array[n7++] = (byte)(this.bitMask & data[n6 + (n8 >> 3)] >> this.shiftOffset - (n8 & 0x7));
                n8 += pixelBitStride;
                ++j;
            }
        }
        return array;
    }
    
    @Override
    public void setDataElements(int n, int n2, Object o) {
        if (n < this.minX || n2 < this.minY || n >= this.maxX || n2 >= this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        byte[] array = (byte[])(byte[])o;
        int n3 = this.dataBitOffset + (n - this.minX) * this.pixelBitStride;
        int n4 = (n2 - this.minY) * this.scanlineStride + (n3 >> 3);
        int n5 = this.shiftOffset - (n3 & 0x7);
        this.data[n4] = (byte)((byte)(this.data[n4] & ~(this.bitMask << n5)) | (array[0] & this.bitMask) << n5);
        this.markDirty();
        return;
    }
    
    @Override
    public void setDataElements(int x, int y, Raster inRaster) {
        if (!(inRaster instanceof BytePackedRaster) || ((BytePackedRaster)inRaster).pixelBitStride != this.pixelBitStride) {
            super.setDataElements(x, y, inRaster);
            return;
        }
        int minX = inRaster.getMinX();
        int minY = inRaster.getMinY();
        int n = minX + x;
        int n2 = minY + y;
        int width = inRaster.getWidth();
        int height = inRaster.getHeight();
        if (n < this.minX || n2 < this.minY || n + width > this.maxX || n2 + height > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        this.setDataElements(n, n2, minX, minY, width, height, (BytePackedRaster)inRaster);
        return;
    }
    
    private void setDataElements(int n, int n2, int n3, int n4, int n5, int n6, BytePackedRaster bytePackedRaster) {
        if (n5 <= 0 || n6 <= 0) {
            return;
        }
        byte[] data = bytePackedRaster.data;
        byte[] data2 = this.data;
        int scanlineStride = bytePackedRaster.scanlineStride;
        int scanlineStride2 = this.scanlineStride;
        int n7 = bytePackedRaster.dataBitOffset + 8 * (n4 - bytePackedRaster.minY) * scanlineStride + (n3 - bytePackedRaster.minX) * bytePackedRaster.pixelBitStride;
        int n8 = this.dataBitOffset + 8 * (n2 - this.minY) * scanlineStride2 + (n - this.minX) * this.pixelBitStride;
        int n9 = n5 * this.pixelBitStride;
        if ((n7 & 0x7) == (n8 & 0x7)) {
            int n38 = n8 & 0x7;
            if (n38 != 0) {
                int n39 = 8 - n38;
                int n40 = n7 >> 3;
                int n41 = n8 >> 3;
                int n42 = 255 >> n38;
                if (n9 < n39) {
                    n42 &= 255 << n39 - n9;
                    n39 = n9;
                }
                for (int n43 = 0; n43 < n6; ++n43) {
                    data2[n41] = (byte)((data2[n41] & ~n42) | (data[n40] & n42));
                    n40 += scanlineStride;
                    n41 += scanlineStride2;
                }
                n7 += n39;
                n8 += n39;
                n9 -= n39;
            }
            if (n9 >= 8) {
                int n44 = n7 >> 3;
                int n45 = n8 >> 3;
                int n46 = n9 >> 3;
                if (n46 == scanlineStride && scanlineStride == scanlineStride2) {
                    System.arraycopy(data, n44, data2, n45, scanlineStride * n6);
                }
                else {
                    for (int n47 = 0; n47 < n6; n44 += scanlineStride, n45 += scanlineStride2, ++n47) {
                        System.arraycopy(data, n44, data2, n45, n46);
                    }
                }
                int n48 = n46 * 8;
                n7 += n48;
                n8 += n48;
                n9 -= n48;
            }
            if (n9 > 0) {
                int n49 = n7 >> 3;
                int n50 = n8 >> 3;
                int n51 = 65280 >> n9 & 0xFF;
                for (int n52 = 0; n52 < n6; ++n52) {
                    data2[n50] = (byte)((data2[n50] & ~n51) | (data[n49] & n51));
                    n49 += scanlineStride;
                    n50 += scanlineStride2;
                }
            }
        }
        else {
            int n10 = n8 & 0x7;
            if (n10 != 0 || n9 < 8) {
                int n11 = 8 - n10;
                int n12 = n7 >> 3;
                int n13 = n8 >> 3;
                int n14 = n7 & 0x7;
                int n15 = 8 - n14;
                int n16 = 255 >> n10;
                if (n9 < n11) {
                    n16 &= 255 << n11 - n9;
                    n11 = n9;
                }
                int n17 = data.length - 1;
                int n18;
                int n19;
                for (int i = 0; i < n6; data2[n13] = (byte)((data2[n13] & ~n16) | ((n18 << n14 | (n19 & 0xFF) >> n15) >> n10 & n16)), n12 += scanlineStride, n13 += scanlineStride2, ++i) {
                    n18 = data[n12];
                    n19 = 0;
                    if (n12 < n17) {
                        n19 = data[n12 + 1];
                    }
                }
                n7 += n11;
                n8 += n11;
                n9 -= n11;
            }
            if (n9 >= 8) {
                int n20 = n7 >> 3;
                int n21 = n8 >> 3;
                int n22 = n9 >> 3;
                int n23 = n7 & 0x7;
                int n24 = 8 - n23;
                for (int j = 0; j < n6; ++j) {
                    int n25 = n20 + j * scanlineStride;
                    int n26 = n21 + j * scanlineStride2;
                    int n27 = data[n25];
                    for (int k = 0; k < n22; ++k) {
                        int n28 = data[n25 + 1];
                        data2[n26] = (byte)(n27 << n23 | (n28 & 0xFF) >> n24);
                        n27 = n28;
                        ++n25;
                        ++n26;
                    }
                }
                int n29 = n22 * 8;
                n7 += n29;
                n8 += n29;
                n9 -= n29;
            }
            if (n9 > 0) {
                int n30 = n7 >> 3;
                int n31 = n8 >> 3;
                int n32 = 65280 >> n9 & 0xFF;
                int n33 = n7 & 0x7;
                int n34 = 8 - n33;
                int n35 = data.length - 1;
                int n36;
                int n37;
                for (int l = 0; l < n6; data2[n31] = (byte)((data2[n31] & ~n32) | ((n36 << n33 | (n37 & 0xFF) >> n34) & n32)), n30 += scanlineStride, n31 += scanlineStride2, ++l) {
                    n36 = data[n30];
                    n37 = 0;
                    if (n30 < n35) {
                        n37 = data[n30 + 1];
                    }
                }
            }
        }
        this.markDirty();
        return;
    }
    
    @Override
    public void setRect(int dx, int dy, Raster srcRaster) {
        if (!(srcRaster instanceof BytePackedRaster) || ((BytePackedRaster)srcRaster).pixelBitStride != this.pixelBitStride) {
            super.setRect(dx, dy, srcRaster);
            return;
        }
        int width = srcRaster.getWidth();
        int height = srcRaster.getHeight();
        int minX = srcRaster.getMinX();
        int minY = srcRaster.getMinY();
        int minX2 = dx + minX;
        int minY2 = dy + minY;
        if (minX2 < this.minX) {
            int n = this.minX - minX2;
            width -= n;
            minX += n;
            minX2 = this.minX;
        }
        if (minY2 < this.minY) {
            int n2 = this.minY - minY2;
            height -= n2;
            minY += n2;
            minY2 = this.minY;
        }
        if (minX2 + width > this.maxX) {
            width = this.maxX - minX2;
        }
        if (minY2 + height > this.maxY) {
            height = this.maxY - minY2;
        }
        this.setDataElements(minX2, minY2, minX, minY, width, height, (BytePackedRaster)srcRaster);
        return;
    }
    
    @Override
    public void setDataElements(int n, int n2, int n3, int n4, Object o) {
        this.putByteData(n, n2, n3, n4, (byte[])(byte[])o);
    }
    
    public void putByteData(int n, int n2, int n3, int n4, int n5, byte[] array) {
        this.putByteData(n, n2, n3, n4, array);
    }
    
    public void putByteData(int n, int n2, int n3, int n4, byte[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        if (n3 == 0 || n4 == 0) {
            return;
        }
        int pixelBitStride = this.pixelBitStride;
        int n5 = this.dataBitOffset + (n - this.minX) * pixelBitStride;
        int n6 = (n2 - this.minY) * this.scanlineStride;
        int n7 = 0;
        byte[] data = this.data;
        for (int i = 0; i < n4; n6 += this.scanlineStride, ++i) {
            int n8;
            int j;
            for (n8 = n5, j = 0; j < n3 && (n8 & 0x7) != 0x0; n8 += pixelBitStride, ++j) {
                int n9 = this.shiftOffset - (n8 & 0x7);
                data[n6 + (n8 >> 3)] = (byte)((data[n6 + (n8 >> 3)] & ~(this.bitMask << n9)) | (array[n7++] & this.bitMask) << n9);
            }
            int n10 = n6 + (n8 >> 3);
            switch (pixelBitStride) {
                case 1: {
                    while (j < n3 - 7) {
                        data[n10++] = (byte)((array[n7++] & 0x1) << 7 | (array[n7++] & 0x1) << 6 | (array[n7++] & 0x1) << 5 | (array[n7++] & 0x1) << 4 | (array[n7++] & 0x1) << 3 | (array[n7++] & 0x1) << 2 | (array[n7++] & 0x1) << 1 | (array[n7++] & 0x1));
                        n8 += 8;
                        j += 8;
                    }
                    break;
                }
                case 2: {
                    while (j < n3 - 7) {
                        data[n10++] = (byte)((array[n7++] & 0x3) << 6 | (array[n7++] & 0x3) << 4 | (array[n7++] & 0x3) << 2 | (array[n7++] & 0x3));
                        data[n10++] = (byte)((array[n7++] & 0x3) << 6 | (array[n7++] & 0x3) << 4 | (array[n7++] & 0x3) << 2 | (array[n7++] & 0x3));
                        n8 += 16;
                        j += 8;
                    }
                    break;
                }
                case 4: {
                    while (j < n3 - 7) {
                        data[n10++] = (byte)((array[n7++] & 0xF) << 4 | (array[n7++] & 0xF));
                        data[n10++] = (byte)((array[n7++] & 0xF) << 4 | (array[n7++] & 0xF));
                        data[n10++] = (byte)((array[n7++] & 0xF) << 4 | (array[n7++] & 0xF));
                        data[n10++] = (byte)((array[n7++] & 0xF) << 4 | (array[n7++] & 0xF));
                        n8 += 32;
                        j += 8;
                    }
                    break;
                }
            }
            while (j < n3) {
                int n11 = this.shiftOffset - (n8 & 0x7);
                data[n6 + (n8 >> 3)] = (byte)((data[n6 + (n8 >> 3)] & ~(this.bitMask << n11)) | (array[n7++] & this.bitMask) << n11);
                n8 += pixelBitStride;
                ++j;
            }
        }
        this.markDirty();
        return;
    }
    
    @Override
    public int[] getPixels(int n, int n2, int n3, int n4, int[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        if (array == null) {
            array = new int[n3 * n4];
        }
        int pixelBitStride = this.pixelBitStride;
        int n5 = this.dataBitOffset + (n - this.minX) * pixelBitStride;
        int n6 = (n2 - this.minY) * this.scanlineStride;
        int n7 = 0;
        byte[] data = this.data;
        for (int i = 0; i < n4; n6 += this.scanlineStride, ++i) {
            int n8;
            int j;
            for (n8 = n5, j = 0; j < n3 && (n8 & 0x7) != 0x0; n8 += pixelBitStride, ++j) {
                array[n7++] = (this.bitMask & data[n6 + (n8 >> 3)] >> this.shiftOffset - (n8 & 0x7));
            }
            int n9 = n6 + (n8 >> 3);
            switch (pixelBitStride) {
                case 1: {
                    while (j < n3 - 7) {
                        int n10 = data[n9++];
                        array[n7++] = (n10 >> 7 & 0x1);
                        array[n7++] = (n10 >> 6 & 0x1);
                        array[n7++] = (n10 >> 5 & 0x1);
                        array[n7++] = (n10 >> 4 & 0x1);
                        array[n7++] = (n10 >> 3 & 0x1);
                        array[n7++] = (n10 >> 2 & 0x1);
                        array[n7++] = (n10 >> 1 & 0x1);
                        array[n7++] = (n10 & 0x1);
                        n8 += 8;
                        j += 8;
                    }
                    break;
                }
                case 2: {
                    while (j < n3 - 7) {
                        int n11 = data[n9++];
                        array[n7++] = (n11 >> 6 & 0x3);
                        array[n7++] = (n11 >> 4 & 0x3);
                        array[n7++] = (n11 >> 2 & 0x3);
                        array[n7++] = (n11 & 0x3);
                        int n12 = data[n9++];
                        array[n7++] = (n12 >> 6 & 0x3);
                        array[n7++] = (n12 >> 4 & 0x3);
                        array[n7++] = (n12 >> 2 & 0x3);
                        array[n7++] = (n12 & 0x3);
                        n8 += 16;
                        j += 8;
                    }
                    break;
                }
                case 4: {
                    while (j < n3 - 7) {
                        int n13 = data[n9++];
                        array[n7++] = (n13 >> 4 & 0xF);
                        array[n7++] = (n13 & 0xF);
                        int n14 = data[n9++];
                        array[n7++] = (n14 >> 4 & 0xF);
                        array[n7++] = (n14 & 0xF);
                        int n15 = data[n9++];
                        array[n7++] = (n15 >> 4 & 0xF);
                        array[n7++] = (n15 & 0xF);
                        int n16 = data[n9++];
                        array[n7++] = (n16 >> 4 & 0xF);
                        array[n7++] = (n16 & 0xF);
                        n8 += 32;
                        j += 8;
                    }
                    break;
                }
            }
            while (j < n3) {
                array[n7++] = (this.bitMask & data[n6 + (n8 >> 3)] >> this.shiftOffset - (n8 & 0x7));
                n8 += pixelBitStride;
                ++j;
            }
        }
        return array;
    }
    
    @Override
    public void setPixels(int n, int n2, int n3, int n4, int[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int pixelBitStride = this.pixelBitStride;
        int n5 = this.dataBitOffset + (n - this.minX) * pixelBitStride;
        int n6 = (n2 - this.minY) * this.scanlineStride;
        int n7 = 0;
        byte[] data = this.data;
        for (int i = 0; i < n4; n6 += this.scanlineStride, ++i) {
            int n8;
            int j;
            for (n8 = n5, j = 0; j < n3 && (n8 & 0x7) != 0x0; n8 += pixelBitStride, ++j) {
                int n9 = this.shiftOffset - (n8 & 0x7);
                data[n6 + (n8 >> 3)] = (byte)((data[n6 + (n8 >> 3)] & ~(this.bitMask << n9)) | (array[n7++] & this.bitMask) << n9);
            }
            int n10 = n6 + (n8 >> 3);
            switch (pixelBitStride) {
                case 1: {
                    while (j < n3 - 7) {
                        data[n10++] = (byte)((array[n7++] & 0x1) << 7 | (array[n7++] & 0x1) << 6 | (array[n7++] & 0x1) << 5 | (array[n7++] & 0x1) << 4 | (array[n7++] & 0x1) << 3 | (array[n7++] & 0x1) << 2 | (array[n7++] & 0x1) << 1 | (array[n7++] & 0x1));
                        n8 += 8;
                        j += 8;
                    }
                    break;
                }
                case 2: {
                    while (j < n3 - 7) {
                        data[n10++] = (byte)((array[n7++] & 0x3) << 6 | (array[n7++] & 0x3) << 4 | (array[n7++] & 0x3) << 2 | (array[n7++] & 0x3));
                        data[n10++] = (byte)((array[n7++] & 0x3) << 6 | (array[n7++] & 0x3) << 4 | (array[n7++] & 0x3) << 2 | (array[n7++] & 0x3));
                        n8 += 16;
                        j += 8;
                    }
                    break;
                }
                case 4: {
                    while (j < n3 - 7) {
                        data[n10++] = (byte)((array[n7++] & 0xF) << 4 | (array[n7++] & 0xF));
                        data[n10++] = (byte)((array[n7++] & 0xF) << 4 | (array[n7++] & 0xF));
                        data[n10++] = (byte)((array[n7++] & 0xF) << 4 | (array[n7++] & 0xF));
                        data[n10++] = (byte)((array[n7++] & 0xF) << 4 | (array[n7++] & 0xF));
                        n8 += 32;
                        j += 8;
                    }
                    break;
                }
            }
            while (j < n3) {
                int n11 = this.shiftOffset - (n8 & 0x7);
                data[n6 + (n8 >> 3)] = (byte)((data[n6 + (n8 >> 3)] & ~(this.bitMask << n11)) | (array[n7++] & this.bitMask) << n11);
                n8 += pixelBitStride;
                ++j;
            }
        }
        this.markDirty();
        return;
    }
    
    @Override
    public Raster createChild(int n, int n2, int n3, int n4, int n5, int n6, int[] array) {
        return this.createWritableChild(n, n2, n3, n4, n5, n6, array);
    }
    
    @Override
    public WritableRaster createWritableChild(int n, int n2, int width, int height, int x, int y, int[] array) {
        if (n < this.minX) {
            throw new RasterFormatException("x lies outside the raster");
        }
        if (n2 < this.minY) {
            throw new RasterFormatException("y lies outside the raster");
        }
        if (n + width < n || n + width > this.minX + this.width) {
            throw new RasterFormatException("(x + width) is outside of Raster");
        }
        if (n2 + height < n2 || n2 + height > this.minY + this.height) {
            throw new RasterFormatException("(y + height) is outside of Raster");
        }
        SampleModel sampleModel;
        if (array != null) {
            sampleModel = this.sampleModel.createSubsetSampleModel(array);
        }
        else {
            sampleModel = this.sampleModel;
        }
        return new BytePackedRaster(sampleModel, this.dataBuffer, new Rectangle(x, y, width, height), new Point(this.sampleModelTranslateX + (x - n), this.sampleModelTranslateY + (y - n2)), this);
    }
    
    @Override
    public WritableRaster createCompatibleWritableRaster(int n, int n2) {
        if (n <= 0 || n2 <= 0) {
            throw new RasterFormatException("negative " + ((n <= 0) ? "width" : "height"));
        }
        return new BytePackedRaster(this.sampleModel.createCompatibleSampleModel(n, n2), new Point(0, 0));
    }
    
    @Override
    public WritableRaster createCompatibleWritableRaster() {
        return this.createCompatibleWritableRaster(this.width, this.height);
    }
    
    private void verify(boolean b) {
        if (this.dataBitOffset < 0) {
            throw new RasterFormatException("Data offsets must be >= 0");
        }
        if (this.width <= 0 || this.height <= 0 || this.height > Integer.MAX_VALUE / this.width) {
            throw new RasterFormatException("Invalid raster dimension");
        }
        if (this.width - 1 > Integer.MAX_VALUE / this.pixelBitStride) {
            throw new RasterFormatException("Invalid raster dimension");
        }
        if ((long)this.minX - (long)this.sampleModelTranslateX < 0L || (long)this.minY - (long)this.sampleModelTranslateY < 0L) {
            throw new RasterFormatException("Incorrect origin/translate: (" + this.minX + ", " + this.minY + ") / (" + this.sampleModelTranslateX + ", " + this.sampleModelTranslateY + ")");
        }
        if (this.scanlineStride < 0 || this.scanlineStride > Integer.MAX_VALUE / this.height) {
            throw new RasterFormatException("Invalid scanline stride");
        }
        if ((this.height > 1 || this.minY - this.sampleModelTranslateY > 0) && this.scanlineStride > this.data.length) {
            throw new RasterFormatException("Incorrect scanline stride: " + this.scanlineStride);
        }
        long n = (long)this.dataBitOffset + (long)(this.height - 1) * (long)this.scanlineStride * 8L + (long)(this.width - 1) * (long)this.pixelBitStride + (long)this.pixelBitStride - 1L;
        if (n < 0L || n / 8L >= (long)this.data.length) {
            throw new RasterFormatException("raster dimensions overflow array bounds");
        }
        if (b && this.height > 1 && (long)(this.width * this.pixelBitStride - 1) / 8L >= (long)this.scanlineStride) {
            throw new RasterFormatException("data for adjacent scanlines overlaps");
        }
        return;
    }
    
    @Override
    public String toString() {
        return new String("BytePackedRaster: width = " + this.width + " height = " + this.height + " #channels " + this.numBands + " xOff = " + this.sampleModelTranslateX + " yOff = " + this.sampleModelTranslateY);
    }
    
    static {
        NativeLibLoader.loadLibraries();
        initIDs();
    }
}
